/**
*
* cycles.cpp
* Copyright © 2017 Wyatt Sheffield
* This work is free. You can redistribute it and/or modify it under the
* terms of the Do What The Fuck You Want To Public License, Version 2,
* as published by Sam Hocevar. See http://www.wtfpl.net/ for more details.
*
*/

#include <iostream>
#include <string>
#include <vector>

//for all a,b>0
//Euclid's algorithm
int gcd(int a, int b){
    if(b == 0){ return a; }
    return gcd(b, a%b);
}

//for all a,b>0
int lcm(int a, int b){
    return (a*b)/gcd(a, b);
}

void printVec(std::vector<std::string> v){
    for(size_t i = 0; i < v.size(); i++){
        std::cout << "Cycle " << i + 1 << " (Length " << v[i].size() << ") :\t" << v[i] << '\n';
    }
}

void capitalize(std::string& in){
	for (size_t i = 0; i < in.length(); i++){
		if (in[i] >= 'a' && in[i] <= 'z'){ in[i] = (in[i] - 'a') + 'A'; }
	}
}

char encodeChar(char c, std::string key){
    return key[c - 'A'];
}

std::vector<std::string> findCycles(std::string key) {
    std::string cycle;
    std::vector<std::string> cycles;
    bool visited[26] = {false};
    char current, trace;
    for(int i = 0; i < 26; i++){
        if(visited[i]) { continue; } //already been there; move on to the next one.
        visited[i] = true;
        cycle = "";
        current = i + 'A';
        cycle += current;
        trace = encodeChar(current, key);
        while(trace != current){ //loop until we are back to the current char
            cycle += trace; //add the last encoded char to the string
            visited[trace - 'A'] = true; //we have just dealt with the trace and do not need to do so again
            trace = encodeChar(trace, key); //reencode the trace. if it comes back to the current char, the loop will exit.
        }
        cycles.push_back(cycle);
    }
    return cycles;
}

int findPeriod(std::vector<std::string> cycles){
    int period = cycles[0].length();
    if(cycles.size() == 1){ return period; }
    for(size_t i = 1; i < cycles.size(); i++){
        period = lcm(period, cycles[i].length());
    }
    return period;
}

int main(int argc, char* argv[]) {
    std::string key = argv[1];
    if(key.length() != 26){
        std::cerr << "The key must contain 26 letters." << std::endl;
        return 1;
    }
    capitalize(key);
    std::vector<std::string> cycles = findCycles(key);
    std::cout << "\nThe key [" << key << "] has the following " << cycles.size() << " cycles:\n\n";
    printVec(cycles);
    std::cout << "\nand a total period of " << findPeriod(cycles) << ".\n\n";
    return 0;
}
